import stat
import subprocess
from typing import List

from debputy import elf_util
from debputy.filesystem_scan import FSPath
from debputy.packages import BinaryPackage

SKIPPED_DEBUG_DIRS = [
    'lib',
    'lib64',
    'usr',
    'bin',
    'sbin',
    'opt',
    'dev',
    'emul',
    '.build-id',
]

SKIP_DIRS = {
    f'./usr/lib/debug/{subdir}'
    for subdir in ['lib', 'lib64', 'usr', 'bin', 'sbin', 'opt', 'dev', 'emul', '.build-id']
}


def _walk_filter(fs_path: FSPath, children: List[FSPath]) -> bool:
    if fs_path.path in SKIP_DIRS:
        children.clear()
        return False
    return True


def find_shlibdeps(binary_package: BinaryPackage,
                   fs_root: FSPath,
                   substvars_file: str):
    elf_files_to_process = elf_util.find_all_so_files(fs_root, walk_filter=_walk_filter)

    if not elf_files_to_process:
        return

    dpkg_cmd = [
        'dpkg-shlibdeps',
        f'-T{substvars_file}'
    ]
    if binary_package.is_udeb:
        dpkg_cmd.append('-tudeb')
    dpkg_cmd.extend(p.fs_path for p in elf_files_to_process)
    print(f"Executing for {binary_package.name}: {dpkg_cmd}")
    subprocess.check_call(dpkg_cmd)


